package errors

import "fmt"

// TypeMismatch represents an error where a type requirement was not met.
type TypeMismatch struct {
	Encountered   string
	Expected      string
	ParameterName string
	IsReturn      bool
}

// Error generates the string representation.
func (err *TypeMismatch) Error() string {
	subject := "type"

	if err.IsReturn {
		subject = "return type"
	}

	if err.ParameterName != "" {
		return fmt.Sprintf("Expected parameter '%s' of %s '%s' (encountered '%s')", err.ParameterName, subject, err.Expected, err.Encountered)
	}

	return fmt.Sprintf("Expected %s '%s' instead of '%s'", subject, err.Expected, err.Encountered)
}
